@@ -147,6 +147,10 @@ create_table_pages_html <- function(rs, cntnt, lpg_rows) {
     control_cols <- c(control_cols, "..stub_var")
   }
   
+  if ("..group_border" %in% names(fdat)){
+    control_cols <- c(control_cols, "..group_border")
+  }
+  
   # Reset keys, since prep_data can add/remove columns for stub
   keys <- names(fdat)
   # print("Keys")
@@ -190,7 +194,7 @@ create_table_pages_html <- function(rs, cntnt, lpg_rows) {
   
   # Create a temporary page info to pass into get_content_offsets
   tmp_pi <- list(keys = keys, col_width = widths_uom, label = labels,
-                 label_align = label_aligns, table_align = cntnt$align)
+                 label_align = label_aligns, table_align = cntnt$align, data = fdat)
   # print("Temp PI")
   # print(tmp_pi)
   
@@ -228,17 +232,16 @@ create_table_pages_html <- function(rs, cntnt, lpg_rows) {
         wrap_flag <- FALSE
       
       #print(s)
-      # Ensure content blank rows are added only to the first and last pages
-      blnk_ind <- get_blank_indicator(counter, tot_count, content_blank_row,
-                                      rs$body_line_count, content_offset$lines, 
-                                      nrow(s))
-      #print(blnk_ind)
-      
       if (!is.na(pgby_var))
         pgby <- trimws(s[1, "..page_by"])
       else 
         pgby <- NULL
       
+      # Ensure content blank rows are added only to the first and last pages
+      blnk_ind <- get_blank_indicator(counter, tot_count, content_blank_row,
+                                      rs$body_line_count, content_offset$lines, 
+                                      nrow(s), pgby)
+      #print(blnk_ind)
       
       pi <- page_info(data= s[, pg], keys = pg, label=labels[pg],
                       col_width = widths_uom[pg], col_align = aligns[pg],
@@ -468,14 +471,45 @@ get_content_offsets_html <- function(rs, ts, pi, content_blank_row, pgby_cnt = N
     ttls <- get_title_header_html(ts$title_hdr, wdth, rs)
   
   # Get page by if it exists
-  pgb <- list(lines = 0, twips = 0)
-  if (!is.null(ts$page_by))
-    pgb <- get_page_by_html(ts$page_by, wdth, NULL, rs, pi$table_align, pgby_cnt = pgby_cnt)
-  else if (!is.null(rs$page_by))
-    pgb <- get_page_by_html(rs$page_by, wdth, NULL, rs, pi$table_align, pgby_cnt = pgby_cnt)
+  # pgb <- list(lines = 0, twips = 0)
+  # if (!is.null(ts$page_by))
+  #   pgb <- get_page_by_html(ts$page_by, wdth, NULL, rs, pi$table_align, pgby_cnt = pgby_cnt)
+  # else if (!is.null(rs$page_by))
+  #   pgb <- get_page_by_html(rs$page_by, wdth, NULL, rs, pi$table_align, pgby_cnt = pgby_cnt)
+  page_by_info <- NULL
+  if (!is.null(ts$page_by)) {
+    page_by_info <- ts$page_by
+  } else if (!is.null(rs$page_by)) {
+    page_by_info <- rs$page_by
+  }
+  
+  # Get lines for each page by value
+  if (!is.null(page_by_info)) {
+    cnt <- c(upper = list(), lower = 0, blank_upper = 0, blank_lower = 0)
+    
+    pgby_unique <- unique(pi$data$..page_by)
+    
+    for (i in 1:length(pgby_unique)) {
+      pgby_temp <- get_page_by_html(page_by_info, wdth, pgby_unique[i], rs, pi$table_align, pgby_cnt = pgby_cnt)
+      
+      # Add everything up
+      cnt[["upper"]][[i]] <- shdrs$lines + hdrs$lines + ttls$lines + pgby_temp$lines
+    }
+    
+    names(cnt[["upper"]]) <- pgby_unique
+    
+  } else {
+    pgb <- list(lines = 0)
+    pgb <- get_page_by_html(page_by_info, wdth, NULL, rs, pi$table_align, pgby_cnt = pgby_cnt)
+    cnt <- c(upper = 0, lower = 0, blank_upper = 0, blank_lower = 0)
+    
+    # Add everything up
+    cnt[["upper"]] <- shdrs$lines + hdrs$lines + ttls$lines + pgb$lines
+    
+  }
   
   # Add everything up
-  cnt[["upper"]] <- shdrs$lines + hdrs$lines + ttls$lines + pgb$lines
+  # cnt[["upper"]] <- shdrs$lines + hdrs$lines + ttls$lines + pgb$lines
   
   if (content_blank_row %in% c("above", "both")) {
     #ret[["blank_upper"]] <- rs$line_height
@@ -1037,10 +1071,20 @@ get_table_body_html <- function(rs, tbl, widths, algns, talgn, tbrdrs,
         
         sflg <- nms[j] == "stub" &  has_style(rs, "table_stub_background")
         
+        cell_border <- NULL
+        if ("..group_border" %in% names(tbl)) {
+          cell_border <- tbl[["..group_border"]][i]
+        }
+        
+        # Don't draw group line for first blank row
+        if (flgs[i] %in% c("B", "A") & i == 1){
+          cell_border <- NULL
+        }
+        
         b <- get_cell_borders_html(i, j, nrow(t), ncol(t), brdrs, flgs[i], 
                                    exclude = exclude_top, 
                                    border_color = get_style(rs, "border_color"),
-                                    stub_flag = sflg)
+                                    stub_flag = sflg, cell_border = cell_border)
         
         # Put indent information into style
         if (rs$units == "inches") {
@@ -1142,7 +1186,7 @@ get_table_body_html <- function(rs, tbl, widths, algns, talgn, tbrdrs,
         }
         
         # Count lines in cell
-        cl <- strsplit(vl, "\n", fixed = TRUE)[[1]]
+        cl <- strsplit(vl, "<br>", fixed = TRUE)[[1]]
         if (length(cl) > mxrw)
           mxrw <- length(cl) 
       }
