@@ -147,6 +147,10 @@ create_table_pages_pdf <- function(rs, cntnt, lpg_rows) {
     control_cols <- c(control_cols, "..stub_var")
   }
   
+  if ("..group_border" %in% names(fdat)) {
+    control_cols <- c(control_cols, "..group_border")
+  }
+  
   # Reset keys, since prep_data can add/remove columns for stub
   keys <- names(fdat)
   # print("Keys")
@@ -188,7 +192,7 @@ create_table_pages_pdf <- function(rs, cntnt, lpg_rows) {
   
   # Create a temporary page info to pass into get_content_offsets
   tmp_pi <- list(keys = keys, col_width = widths_uom, label = labels,
-                 label_align = label_aligns, table_align = cntnt$align)
+                 label_align = label_aligns, table_align = cntnt$align, data = fdat)
   # print("Temp PI")
   # print(tmp_pi)
   
@@ -237,17 +241,16 @@ create_table_pages_pdf <- function(rs, cntnt, lpg_rows) {
         wrap_flag <- FALSE
       
       #print(s)
-      # Ensure content blank rows are added only to the first and last pages
-      blnk_ind <- get_blank_indicator(counter, tot_count, content_blank_row,
-                                      rs$body_line_count, content_offset$lines, 
-                                      nrow(s))
-      #print(blnk_ind)
-      
       if (!is.na(pgby_var))
         pgby <- trimws(s[1, "..page_by"])
       else 
         pgby <- NULL
       
+      # Ensure content blank rows are added only to the first and last pages
+      blnk_ind <- get_blank_indicator(counter, tot_count, content_blank_row,
+                                      rs$body_line_count, content_offset$lines, 
+                                      nrow(s), pgby)
+      #print(blnk_ind)
       
       pi <- page_info(data= s[, pg], keys = pg, label=labels[pg],
                       col_width = widths_uom[pg], col_align = aligns[pg],
@@ -523,11 +526,10 @@ get_content_offsets_pdf <- function(rs, ts, pi, content_blank_row, pgby_cnt = NU
     ttls <- get_title_header_pdf(ts$title_hdr, wdth, rs)
   
   # Get page by if it exists
-  pgb <- list(lines = 0, points = 0)
-  if (!is.null(ts$page_by))
-    pgb <- get_page_by_pdf(ts$page_by, wdth, NULL, rs, pi$table_align, pgby_cnt = pgby_cnt)
-  else if (!is.null(rs$page_by))
-    pgb <- get_page_by_pdf(rs$page_by, wdth, NULL, rs, pi$table_align, pgby_cnt = pgby_cnt)
+  # if (!is.null(ts$page_by))
+  #   pgb <- get_page_by_pdf(ts$page_by, wdth, NULL, rs, pi$table_align, pgby_cnt = pgby_cnt)
+  # else if (!is.null(rs$page_by))
+  #   pgb <- get_page_by_pdf(rs$page_by, wdth, NULL, rs, pi$table_align, pgby_cnt = pgby_cnt)
   
   
   #print(length(pgb))
@@ -535,8 +537,45 @@ get_content_offsets_pdf <- function(rs, ts, pi, content_blank_row, pgby_cnt = NU
   # print(paste("Table titles:", ttls))
   
   # Add everything up
-  ret[["upper"]] <- shdrs$points + hdrs$points + ttls$points + pgb$points
-  cnt[["upper"]] <- shdrs$lines + hdrs$lines + ttls$lines + pgb$lines
+  # ret[["upper"]] <- shdrs$points + hdrs$points + ttls$points + pgb$points
+  # cnt[["upper"]] <- shdrs$lines + hdrs$lines + ttls$lines + pgb$lines
+  
+  page_by_info <- NULL
+  if (!is.null(ts$page_by)) {
+    page_by_info <- ts$page_by
+  } else if (!is.null(rs$page_by)) {
+    page_by_info <- rs$page_by
+  }
+  
+  # Get lines for each page by value
+  if (!is.null(page_by_info)) {
+    ret <- c(upper = list(), lower = 0, blank_upper = 0, blank_lower = 0)
+    cnt <- c(upper = list(), lower = 0, blank_upper = 0, blank_lower = 0)
+    
+    pgby_unique <- unique(pi$data$..page_by)
+    
+    for (i in 1:length(pgby_unique)) {
+      pgby_temp <- get_page_by_pdf(page_by_info, wdth, pgby_unique[i], rs, pi$table_align, pgby_cnt = pgby_cnt)
+      
+      # Add everything up
+      ret[["upper"]][[i]] <- shdrs$points + hdrs$points + ttls$points + pgby_temp$points
+      cnt[["upper"]][[i]] <- shdrs$lines + hdrs$lines + ttls$lines + pgby_temp$lines
+    }
+    
+    names(ret[["upper"]]) <- pgby_unique
+    names(cnt[["upper"]]) <- pgby_unique
+    
+  } else {
+    pgb <- list(lines = 0, points = 0)
+    pgb <- get_page_by_pdf(page_by_info, wdth, NULL, rs, pi$table_align, pgby_cnt = pgby_cnt)
+    ret <- c(upper = 0, lower = 0, blank_upper = 0, blank_lower = 0)
+    cnt <- c(upper = 0, lower = 0, blank_upper = 0, blank_lower = 0)
+    
+    # Add everything up
+    ret[["upper"]] <- shdrs$points + hdrs$points + ttls$points + pgb$points
+    cnt[["upper"]] <- shdrs$lines + hdrs$lines + ttls$lines + pgb$lines
+    
+  }
   
   if (content_blank_row %in% c("above", "both")) {
     ret[["blank_upper"]] <- rs$line_height
@@ -559,9 +598,29 @@ get_content_offsets_pdf <- function(rs, ts, pi, content_blank_row, pgby_cnt = NU
   # Not perfect but good enough
   brdrs <- strip_borders(ts$borders )
   if (any(brdrs %in% c("all", "inside"))) {
-    epnts <- floor(rs$body_line_count - cnt[["lower"]] - cnt[["upper"]]) * rs$border_height 
-    ret[["lower"]] <- ret[["lower"]] + epnts - rs$line_height
-    cnt[["lower"]] <- cnt[["lower"]] + floor(epnts / rs$row_height) - 1
+    if (is.null(page_by_info)) {
+      
+      epnts <- floor(rs$body_line_count - cnt[["lower"]] - cnt[["upper"]]) * rs$border_height 
+      ret[["lower"]] <- ret[["lower"]] + epnts - rs$line_height
+      cnt[["lower"]] <- cnt[["lower"]] + floor(epnts / rs$row_height) - 1
+      
+    } else {
+      
+      # Convert lower to list because of different upper
+      ret_lower <- ret[["lower"]]
+      cnt_lower <- cnt[["lower"]]
+      ret[["lower"]] <- list()
+      cnt[["lower"]] <- list()
+      
+      for (i in 1:length(pgby_unique)) {
+        epnts <- floor(rs$body_line_count - cnt_lower - cnt[["upper"]][[i]]) * rs$border_height 
+        ret[["lower"]][[i]] <- ret_lower + epnts - rs$line_height
+        cnt[["lower"]][[i]] <- cnt_lower + floor(epnts / rs$row_height) - 1
+      }
+      
+      names(ret[["lower"]]) <- pgby_unique
+      names(cnt[["lower"]]) <- pgby_unique
+    }
   }
   
   if (content_blank_row %in% c("both", "below")) {
@@ -1258,10 +1317,25 @@ get_table_body_pdf <- function(rs, tbl, widths, algns, talgn, tbrdrs,
       
     }
 
+    group_border <- ""
+    if ("..group_border" %in% names(tbl)) {
+      group_border <- tbl[["..group_border"]][i]
+    }
     
+    # Don't draw group line for first blank row
+    if (blnks[i] %in% c("B", "A") & i == 1){
+      group_border <- ""
+    }
+    
+    # Don't draw group line for last record when table border is all, bottom, outside
+    if (i == nrow(t) & any(brdrs %in% c("all", "outside", "bottom"))) {
+      group_border <- ""
+    }
     
     # Applies inside horizontal borders
-    if (any(brdrs %in% c("all", "inside")) & i < nrow(t)) {
+    if ((any(brdrs %in% c("all", "inside")) & i < nrow(t)) |
+        group_border == "bottom"
+        ) {
     
       ret[[length(ret) + 1]] <- page_hline(tlb * conv, mxrw -rh + bs, 
                                            (trb - tlb) * conv)
@@ -1269,7 +1343,11 @@ get_table_body_pdf <- function(rs, tbl, widths, algns, talgn, tbrdrs,
     }
     
     rline <- mxrw
-  
+    
+    # Add border height if the line is drawn by group border
+    if (!any(brdrs %in% c("all", "inside")) & group_border == "bottom") {
+      rline <- rline + bh
+    }
   }
   
   dev.off()
